@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Salvador E. Tropea
-# Copyright (c) 2020 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2021 Salvador E. Tropea
+# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
 # Copyright (c) 2018 John Beard
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
@@ -9,7 +9,7 @@
 
 Usage:
   kibot [-b BOARD] [-e SCHEMA] [-c CONFIG] [-d OUT_DIR] [-s PRE]
-         [-q | -v...] [-i] [-g DEF]... [TARGET...]
+         [-q | -v...] [-i] [-m MKFILE] [-g DEF]... [TARGET...]
   kibot [-v...] [-c PLOT_CONFIG] --list
   kibot [-v...] [-b BOARD] [-d OUT_DIR] [-p | -P] --example
   kibot [-v...] --help-filters
@@ -37,27 +37,30 @@ Options:
   --help-preflights                List supported preflights and details
   -i, --invert-sel                 Generate the outputs not listed as targets
   -l, --list                       List available outputs (in the config file)
+  -m MKFILE, --makefile MKFILE     Generate a Makefile (no targets created)
   -p, --copy-options               Copy plot options from the PCB file
   -P, --copy-and-expand            As -p but expand the list of layers
   -q, --quiet                      Remove information logs
   -s PRE, --skip-pre PRE           Skip preflights, comma separated or `all`
   -v, --verbose                    Show debugging information
   -V, --version                    Show program's version number and exit
-  -x, --example                    Create an example configuration file.
+  -x, --example                    Create a template configuration file.
 
 """
 __author__ = 'Salvador E. Tropea, John Beard'
-__copyright__ = 'Copyright 2018-2020, Salvador E. Tropea/INTI/John Beard'
+__copyright__ = 'Copyright 2018-2021, Salvador E. Tropea/INTI/John Beard'
 __credits__ = ['Salvador E. Tropea', 'John Beard']
 __license__ = 'GPL v3+'
-__email__ = 'set@ieee.org'
+__email__ = 'stropea@inti.gob.ar'
 __url__ = 'https://github.com/INTI-CMNB/KiBot/'
 __status__ = 'stable'
-__version__ = '0.8.0'
+__version__ = '0.10.1'
 
 
 import os
 import sys
+from sys import path as sys_path
+import re
 import gzip
 import locale
 from glob import glob
@@ -67,13 +70,14 @@ from logging import DEBUG
 from . import log
 log.set_domain('kibot')
 logger = log.init()
+from .docopt import docopt
 from .gs import (GS)
-from .kiplot import (generate_outputs, load_actions, config_output)
+from .misc import (NO_PCB_FILE, NO_SCH_FILE, EXIT_BAD_ARGS, W_VARSCH, W_VARCFG, W_VARPCB, NO_PCBNEW_MODULE,
+                   KICAD_VERSION_5_99, W_NOKIVER, hide_stderr)
 from .pre_base import (BasePreFlight)
 from .config_reader import (CfgYamlReader, print_outputs_help, print_output_help, print_preflights_help, create_example,
                             print_filters_help)
-from .misc import (NO_PCB_FILE, NO_SCH_FILE, EXIT_BAD_ARGS, W_VARSCH, W_VARCFG, W_VARPCB)
-from .docopt import docopt
+from .kiplot import (generate_outputs, load_actions, config_output, generate_makefile)
 
 
 def list_pre_and_outs(logger, outputs):
@@ -86,7 +90,7 @@ def list_pre_and_outs(logger, outputs):
     if len(outputs):
         logger.info('Outputs:')
         for o in outputs:
-            config_output(o)
+            config_output(o, dry=True)
             logger.info('- '+str(o))
 
 
@@ -100,7 +104,7 @@ def solve_schematic(a_schematic, a_board_file):
         else:
             sch = base+'.kicad_sch'
             if os.path.isfile(sch):
-                schematic = sch
+                schematic = sch  # pragma: no cover (Ki6)
     if not schematic:
         schematics = glob('*.sch')+glob('*.kicad_sch')
         if len(schematics) == 1:
@@ -194,6 +198,70 @@ def set_locale():
         pass
 
 
+def detect_kicad():
+    # Check if we have to run the nightly KiCad build
+    nightly = False
+    if os.environ.get('KIAUS_USE_NIGHTLY'):  # pragma: no cover (Ki6)
+        # Path to the Python module
+        sys_path.insert(0, '/usr/lib/kicad-nightly/lib/python3/dist-packages')
+        nightly = True
+    try:
+        import pcbnew
+    except ImportError:
+        logger.error("Failed to import pcbnew Python module."
+                     " Is KiCad installed?"
+                     " Do you need to add it to PYTHONPATH?")
+        sys.exit(NO_PCBNEW_MODULE)
+    try:
+        GS.kicad_version = pcbnew.GetBuildVersion()
+    except AttributeError:
+        logger.warning(W_NOKIVER+"Unknown KiCad version, please install KiCad 5.1.6 or newer")
+        # Assume the best case
+        GS.kicad_version = '5.1.5'
+    m = re.search(r'(\d+)\.(\d+)\.(\d+)', GS.kicad_version)
+    GS.kicad_version_major = int(m.group(1))
+    GS.kicad_version_minor = int(m.group(2))
+    GS.kicad_version_patch = int(m.group(3))
+    GS.kicad_version_n = GS.kicad_version_major*1000000+GS.kicad_version_minor*1000+GS.kicad_version_patch
+    logger.debug('Detected KiCad v{}.{}.{} ({} {})'.format(GS.kicad_version_major, GS.kicad_version_minor,
+                 GS.kicad_version_patch, GS.kicad_version, GS.kicad_version_n))
+    # Used to look for plug-ins.
+    # KICAD_PATH isn't good on my system.
+    # The kicad-nightly package overwrites the regular package!!
+    GS.kicad_share_path = '/usr/share/kicad'
+    if GS.kicad_version_n >= KICAD_VERSION_5_99:  # pragma: no cover (Ki6)
+        GS.kicad_conf_path = pcbnew.GetSettingsManager().GetUserSettingsPath()
+        if nightly:
+            # Nightly Debian packages uses `/usr/share/kicad-nightly/kicad-nightly.env` as an environment extension
+            # This script defines KICAD_CONFIG_HOME="$HOME/.config/kicadnightly"
+            # So we just patch it, as we patch the name of the binaries
+            GS.kicad_conf_path = GS.kicad_conf_path.replace('/kicad/', '/kicadnightly/')
+            GS.kicad_share_path = GS.kicad_share_path.replace('/kicad/', '/kicadnightly/')
+    else:
+        # Bug in KiCad (#6989), prints to stderr:
+        # `../src/common/stdpbase.cpp(62): assert "traits" failed in Get(test_dir): create wxApp before calling this`
+        # Found in KiCad 5.1.8, 5.1.9
+        # So we temporarily supress stderr
+        with hide_stderr():
+            GS.kicad_conf_path = pcbnew.GetKicadConfigPath()
+    # Dirs to look for plugins
+    GS.kicad_plugins_dirs = []
+    # /usr/share/kicad/*
+    GS.kicad_plugins_dirs.append(os.path.join(GS.kicad_share_path, 'scripting'))
+    GS.kicad_plugins_dirs.append(os.path.join(GS.kicad_share_path, 'scripting', 'plugins'))
+    # ~/.config/kicad/*
+    GS.kicad_plugins_dirs.append(os.path.join(GS.kicad_conf_path, 'scripting'))
+    GS.kicad_plugins_dirs.append(os.path.join(GS.kicad_conf_path, 'scripting', 'plugins'))
+    # ~/.kicad_plugins and ~/.kicad
+    if 'HOME' in os.environ:
+        home = os.environ['HOME']
+        GS.kicad_plugins_dirs.append(os.path.join(home, '.kicad_plugins'))
+        GS.kicad_plugins_dirs.append(os.path.join(home, '.kicad', 'scripting'))
+        GS.kicad_plugins_dirs.append(os.path.join(home, '.kicad', 'scripting', 'plugins'))
+    if GS.debug_level > 1:
+        logger.debug('KiCad config path {}'.format(GS.kicad_conf_path))
+
+
 def main():
     set_locale()
     ver = 'KiBot '+__version__+' - '+__copyright__+' - License: '+__license__
@@ -203,6 +271,8 @@ def main():
     log.set_verbosity(logger, args.verbose, args.quiet)
     GS.debug_enabled = logger.getEffectiveLevel() <= DEBUG
     GS.debug_level = args.verbose
+    # Now we have the debug level set we can check (and optionally inform) KiCad info
+    detect_kicad()
 
     # Parse global overwrite options
     for redef in args.global_redef:
@@ -262,8 +332,12 @@ def main():
     GS.set_sch(solve_schematic(args.schematic, args.board_file))
     # Determine the PCB file
     GS.set_pcb(solve_board_file(GS.sch_file, args.board_file))
-    # Do all the job (pre-flight + outputs)
-    generate_outputs(outputs, args.target, args.invert_sel, args.skip_pre)
+    if args.makefile:
+        # Only create a makefile
+        generate_makefile(args.makefile, plot_config, outputs)
+    else:
+        # Do all the job (pre-flight + outputs)
+        generate_outputs(outputs, args.target, args.invert_sel, args.skip_pre)
     # Print total warnings
     logger.log_totals()
 
