@@ -101,32 +101,35 @@ def compare_components(c1, c2, cfg):
 
 class Joiner:
     def __init__(self):
-        self.stack = []
+        self.stack = {}
 
     def add(self, P, N):
-        if not self.stack:
-            self.stack.append(((P, N), (P, N)))
+        if P not in self.stack:
+            self.stack[P] = [((P, N), (P, N))]
             return
-        S, E = self.stack[-1]
+        stack = self.stack[P]
+        S, E = stack[-1]
         if N == E[1] + 1:
-            self.stack[-1] = (S, (P, N))
+            stack[-1] = (S, (P, N))
         else:
-            self.stack.append(((P, N), (P, N)))
+            stack.append(((P, N), (P, N)))
 
     def flush(self, sep, dash='-'):
         refstr = u''
         c = 0
-        for Q in self.stack:
-            if c != 0:
-                refstr += sep
-            S, E = Q
-            if S == E:
-                refstr += "%s%d" % S
-                c += 1
-            else:
-                # Do we have space?
-                refstr += "%s%d%s%s%d" % (S[0], S[1], dash, E[0], E[1])
-                c += 2
+        for pref in sorted(self.stack.keys()):
+            stack = self.stack[pref]
+            for Q in stack:
+                if c != 0:
+                    refstr += sep
+                S, E = Q
+                if S == E:
+                    refstr += "%s%d" % S
+                    c += 1
+                else:
+                    # Do we have space?
+                    refstr += "%s%d%s%s%d" % (S[0], S[1], dash, E[0], E[1])
+                    c += 2
         return refstr
 
 
@@ -151,7 +154,7 @@ class ComponentGroup(object):
 
     def contains_component(self, c):
         """ Test if a given component is already contained in this group """
-        return c.ref in self.refs
+        return c.ref+c.project in self.refs
 
     def add_component(self, c):
         """ Add a component to the group.
@@ -159,15 +162,47 @@ class ComponentGroup(object):
             Note: repeated components happend when a component contains more than one unit """
         if not self.components:
             self.components.append(c)
-            self.refs[c.ref] = c
+            self.refs[c.ref+c.project] = c
         elif self.contains_component(c):
             return
         elif self.match_component(c):
             self.components.append(c)
-            self.refs[c.ref] = c
+            self.refs[c.ref+c.project] = c
 
-    def get_count(self):
-        return len(self.components)
+    def get_count(self, project=None):
+        if project is None:
+            # Total components
+            return len(self.components)
+        # Only for the specified project
+        return sum(map(lambda c: c.project == project, self.components))
+
+    def get_build_count(self):
+        if not self.is_fitted():
+            # Not fitted -> 0
+            return 0
+        if len(self.cfg.aggregate) == 1:
+            # Just one project
+            return len(self.components)*self.cfg.number
+        # Multiple projects, count them using the number of board for each project
+        return sum(map(lambda c: self.cfg.qtys[c.project], self.components))
+
+    def get_sources(self):
+        sources = {}
+        for c in self.components:
+            prj = c.project
+            if self.cfg.source_by_id:
+                prj = self.cfg.source_to_id[prj]
+            if prj in sources:
+                sources[prj] += 1
+            else:
+                sources[prj] = 1
+        field = ''
+        for prj in sorted(sources.keys()):
+            n = sources[prj]
+            if len(field):
+                field += ' '
+            field += prj+'('+str(n)+')'
+        return field
 
     def is_fitted(self):
         # compare_components ensures all has the same status
@@ -194,11 +229,18 @@ class ComponentGroup(object):
 
     def get_alt_refs(self):
         """ Alternative list of references using ranges """
-        S = Joiner()
-        for n in self.components:
-            P, N = (n.ref_prefix, _suffix_to_num(n.ref_suffix))
-            S.add(P, N)
-        return S.flush(' ')
+        refs = ''
+        for sch in self.cfg.aggregate:
+            S = Joiner()
+            for n in self.components:
+                if n.project == sch.name:
+                    S.add(n.ref_id+n.ref_prefix, _suffix_to_num(n.ref_suffix))
+            result = S.flush(self.cfg.ref_separator)
+            if result:
+                if refs:
+                    refs += self.cfg.ref_separator
+                refs += result
+        return refs
 
     def update_field(self, field, value, ref=None):
         """ Update a given field, concatenates existing values and informs a collision """
@@ -214,7 +256,7 @@ class ComponentGroup(object):
         else:
             # Config contains variant information, which is different for each component
             # Part can be one of the defined aliases
-            if field != self.cfg.fit_field and field != 'part':
+            if field not in self.cfg.no_conflict:
                 logger.warning(W_FIELDCONF + "Field conflict: ({refs}) [{name}] : '{flds}' <- '{fld}' (in {ref})".format(
                     refs=self.get_refs(),
                     name=field,
@@ -232,13 +274,18 @@ class ComponentGroup(object):
         else:
             self.fields[ColumnList.COL_REFERENCE_L] = self.get_refs()
         # Quantity
-        q = self.get_count()
-        self.fields[ColumnList.COL_GRP_QUANTITY_L] = "{n}{dnf}{dnc}".format(
-            n=q,
-            dnf=" (DNF)" if not self.is_fitted() else "",
-            dnc=" (DNC)" if self.is_fixed() else "")
-
-        self.fields[ColumnList.COL_GRP_BUILD_QUANTITY_L] = str(q * self.cfg.number) if self.is_fitted() else "0"
+        self.fields[ColumnList.COL_GRP_QUANTITY_L] = str(self.get_count())
+        self.total = self.get_build_count()
+        self.fields[ColumnList.COL_GRP_BUILD_QUANTITY_L] = str(self.total)
+        self.fields[ColumnList.COL_SOURCE_BOM_L] = self.get_sources()
+        # Group status
+        status = ' '
+        if not self.is_fitted():
+            status += '(DNF)'
+        if self.is_fixed():
+            status += '(DNC)'
+        self.fields[ColumnList.COL_STATUS_L] = status
+        # Component data
         comp = self.components[0]
         self.fields[ColumnList.COL_VALUE_L] = comp.value
         self.fields[ColumnList.COL_PART_L] = comp.name
@@ -297,6 +344,25 @@ def normalize_value(c, decimal_point):
     return '{} {}{}'.format(value, mult_s, unit)
 
 
+def compute_multiple_stats(cfg, groups):
+    for sch in cfg.aggregate:
+        sch.comp_total = 0
+        sch.comp_fitted = 0
+        sch.comp_build = 0
+        sch.comp_groups = 0
+        for g in groups:
+            g_l = g.get_count(sch.name)
+            if g_l:
+                sch.comp_groups = sch.comp_groups+1
+            sch.comp_total += g_l
+            if g.is_fitted():
+                sch.comp_fitted += g_l
+        sch.comp_build = sch.comp_fitted*sch.number
+        if cfg.debug_level > 1:
+            logger.debug('Stats for {}: total {} fitted {} build {}'.
+                         format(sch.name, sch.comp_total, sch.comp_fitted, sch.comp_build))
+
+
 def group_components(cfg, components):
     groups = []
     # Iterate through each component, and test whether a group for these already exists
@@ -339,6 +405,7 @@ def group_components(cfg, components):
     # Enumerate the groups and compute stats
     n_total = 0
     n_fitted = 0
+    n_build = 0
     c = 1
     dnf = 1
     cfg.n_groups = len(groups)
@@ -355,9 +422,15 @@ def group_components(cfg, components):
         n_total += g_l
         if is_fitted:
             n_fitted += g_l
+            n_build += g.total
     cfg.n_total = n_total
     cfg.n_fitted = n_fitted
-    cfg.n_build = n_fitted * cfg.number
+    cfg.n_build = n_build
+    if cfg.debug_level > 1:
+        logger.debug('Global stats: total {} fitted {} build {}'.format(n_total, n_fitted, n_build))
+    # Compute stats for multiple schematics
+    if len(cfg.aggregate) > 1:
+        compute_multiple_stats(cfg, groups)
     return groups
 
 
@@ -366,4 +439,7 @@ def do_bom(file_name, ext, comps, cfg):
     groups = group_components(cfg, comps)
     # Create the BoM
     logger.debug("Saving BOM File: "+file_name)
+    number = cfg.number
+    cfg.number = sum(map(lambda prj: prj.number, cfg.aggregate))
     write_bom(file_name, ext, groups, cfg.columns, cfg)
+    cfg.number = number
