@@ -41,18 +41,24 @@ class ColumnList:
     COL_SHEETPATH_L = COL_SHEETPATH.lower()
     COL_ROW_NUMBER = 'Row'
     COL_ROW_NUMBER_L = COL_ROW_NUMBER.lower()
+    COL_STATUS = 'Status'
+    COL_STATUS_L = COL_STATUS.lower()
 
     # Default columns for groups
     COL_GRP_QUANTITY = 'Quantity Per PCB'
     COL_GRP_QUANTITY_L = COL_GRP_QUANTITY.lower()
     COL_GRP_BUILD_QUANTITY = 'Build Quantity'
     COL_GRP_BUILD_QUANTITY_L = COL_GRP_BUILD_QUANTITY.lower()
+    COL_SOURCE_BOM = 'Source BoM'
+    COL_SOURCE_BOM_L = COL_SOURCE_BOM.lower()
 
     # Generated columns
     COLUMNS_GEN_L = {
         COL_GRP_QUANTITY_L: 1,
         COL_GRP_BUILD_QUANTITY_L: 1,
         COL_ROW_NUMBER_L: 1,
+        COL_STATUS_L: 1,
+        COL_SOURCE_BOM_L: 1,
     }
 
     # Default columns
@@ -67,8 +73,10 @@ class ColumnList:
         COL_FP_LIB,
         COL_GRP_QUANTITY,
         COL_GRP_BUILD_QUANTITY,
+        COL_STATUS,
         COL_DATASHEET,
-        COL_SHEETPATH
+        COL_SHEETPATH,
+        COL_SOURCE_BOM,
     ]
 
     # Default columns
