@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Salvador E. Tropea
-# Copyright (c) 2020 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2021 Salvador E. Tropea
+# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
 # Copyright (c) 2018 John Beard
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
@@ -15,7 +15,7 @@ from collections import OrderedDict
 
 from .error import (KiPlotConfigurationError, config_error)
 from .kiplot import (load_board)
-from .misc import (NO_YAML_MODULE, EXIT_BAD_ARGS, EXAMPLE_CFG, WONT_OVERWRITE)
+from .misc import (NO_YAML_MODULE, EXIT_BAD_ARGS, EXAMPLE_CFG, WONT_OVERWRITE, W_NOOUTPUTS)
 from .gs import GS
 from .registrable import RegOutput, RegVariant, RegFilter
 from .pre_base import BasePreFlight
@@ -50,12 +50,16 @@ class CfgYamlReader(object):
 
     def _parse_output(self, o_tree):
         try:
-            name = o_tree['name']
+            name = str(o_tree['name'])
+            if not name:
+                raise KeyError
         except KeyError:
             config_error("Output needs a name in: "+str(o_tree))
 
         try:
             otype = o_tree['type']
+            if not otype:
+                raise KeyError
         except KeyError:
             config_error("Output `"+name+"` needs a type")
 
@@ -63,6 +67,8 @@ class CfgYamlReader(object):
             comment = o_tree['comment']
         except KeyError:
             comment = ''
+        if comment is None:
+            comment = ''
 
         name_type = "`"+name+"` ("+otype+")"
 
@@ -92,13 +98,15 @@ class CfgYamlReader(object):
     def _parse_variant(self, o_tree, kind, reg_class):
         kind_f = kind[0].upper()+kind[1:]
         try:
-            name = o_tree['name']
+            name = str(o_tree['name'])
             if not name:
                 raise KeyError
         except KeyError:
             config_error(kind_f+" needs a name in: "+str(o_tree))
         try:
             otype = o_tree['type']
+            if not otype:
+                raise KeyError
         except KeyError:
             config_error(kind_f+" `"+name+"` needs a type")
         # Is a valid type?
@@ -154,7 +162,7 @@ class CfgYamlReader(object):
         """ Get global options """
         logger.debug("Parsing global options: {}".format(gb))
         if not isinstance(gb, dict):
-            config_error("Incorrect `global` section")
+            config_error("Incorrect `global` section (must be a dict)")
         # Parse all keys inside it
         glb = GS.global_opts_class()
         glb.set_tree(gb)
@@ -163,19 +171,49 @@ class CfgYamlReader(object):
         except KiPlotConfigurationError as e:
             config_error("In `global` section: "+str(e))
 
+    def _parse_import(self, imp, name):
+        """ Get imports """
+        logger.debug("Parsing imports: {}".format(imp))
+        if not isinstance(imp, list):
+            config_error("Incorrect `import` section (must be a list)")
+        # Import the files
+        dir = os.path.dirname(os.path.abspath(name))
+        outputs = []
+        for fn in imp:
+            if not isinstance(fn, str):
+                config_error("`import` items must be strings ({})".format(str(fn)))
+            if not os.path.isabs(fn):
+                fn = os.path.join(dir, fn)
+            if not os.path.isfile(fn):
+                config_error("missing import file `{}`".format(fn))
+            data = self.load_yaml(open(fn))
+            if 'outputs' in data:
+                outs = self._parse_outputs(data['outputs'])
+                outputs.extend(outs)
+                logger.debug('Outputs loaded from `{}`: {}'.format(os.path.relpath(fn), list(map(lambda c: c.name, outs))))
+            else:
+                logger.warning(W_NOOUTPUTS+"No outputs found in `{}`".format(fn))
+        return outputs
+
+    def load_yaml(self, fstream):
+        try:
+            data = yaml.safe_load(fstream)
+        except yaml.YAMLError as e:
+            config_error("Error loading YAML "+str(e))
+        return data
+
     def read(self, fstream):
         """
         Read a file object into a config object
 
         :param fstream: file stream of a config YAML file
         """
-        try:
-            data = yaml.safe_load(fstream)
-        except yaml.YAMLError as e:
-            config_error("Error loading YAML "+str(e))
+        data = self.load_yaml(fstream)
         # Transfer command line global overwrites
         GS.global_output = GS.global_from_cli.get('output', None)
         GS.global_variant = GS.global_from_cli.get('variant', None)
+        GS.global_kiauto_wait_start = GS.global_from_cli.get('kiauto_wait_start', None)
+        GS.global_kiauto_time_out_scale = GS.global_from_cli.get('kiauto_time_out_scale', None)
         # List of outputs
         outputs = []
         version = None
@@ -188,12 +226,14 @@ class CfgYamlReader(object):
                 self._parse_preflight(v)
             elif k == 'global':
                 self._parse_global(v)
+            elif k == 'import':
+                outputs.extend(self._parse_import(v, fstream.name))
             elif k == 'variants':
                 RegOutput.set_variants(self._parse_variants(v))
             elif k == 'filters':
                 RegOutput.set_filters(self._parse_filters(v))
             elif k == 'outputs':
-                outputs = self._parse_outputs(v)
+                outputs.extend(self._parse_outputs(v))
             else:
                 config_error('Unknown section `{}` in config.'.format(k))
         if version is None:
@@ -203,6 +243,8 @@ class CfgYamlReader(object):
 
 def trim(docstring):
     """ PEP 257 recommended trim for __doc__ """
+    if docstring is None:
+        return []
     # Convert tabs to spaces (following the normal Python rules)
     # and split into a list of lines:
     lines = docstring.expandtabs().splitlines()
@@ -244,7 +286,7 @@ def print_output_options(name, cl, indent):
         else:
             entry = '  - `{}`: '
         if help is None:
-            help = 'Undocumented'  # pragma: no cover
+            help = 'Undocumented'
         lines = help.split('\n')
         preface = ind_str+entry.format(k)
         clines = len(lines)
@@ -270,7 +312,7 @@ def print_output_options(name, cl, indent):
 def print_one_out_help(details, n, o):
     lines = trim(o.__doc__)
     if len(lines) == 0:
-        lines = ['Undocumented', 'No description']  # pragma: no cover
+        lines = ['Undocumented', 'No description']
     if details:
         print('* '+lines[0])
         print('  * Type: `{}`'.format(n))
@@ -306,7 +348,7 @@ def print_preflights_help():
     for n, o in OrderedDict(sorted(pres.items())).items():
         help, options = o.get_doc()
         if help is None:
-            help = 'Undocumented'  # pragma: no cover
+            help = 'Undocumented'
         print('- {}: {}.'.format(n, help.strip()))
         if options:
             print_output_options(n, options, 2)
@@ -319,14 +361,15 @@ def print_filters_help():
     for n, o in OrderedDict(sorted(fils.items())).items():
         help = o.__doc__
         if help is None:
-            help = 'Undocumented'  # pragma: no cover
+            help = 'Undocumented'
         print('- {}: {}.'.format(n, help.strip()))
         print_output_options(n, o, 2)
 
 
-def print_example_options(f, cls, name, indent, po):
+def print_example_options(f, cls, name, indent, po, is_list=False):
     ind_str = indent*' '
     obj = cls()
+    first = True
     if po:
         obj.read_vals_from_po(po)
     for k, v in obj.get_attrs_gen():
@@ -346,7 +389,11 @@ def print_example_options(f, cls, name, indent, po):
                             break
                     hl = ' '*i+hl[i:]
                 f.write(ind_str+'# '+hl+'\n')
-        val = getattr(obj, k)
+        example_attr = '_'+k+'_example'
+        if hasattr(obj, example_attr):
+            val = getattr(obj, example_attr)
+        else:
+            val = getattr(obj, k)
         if isinstance(val, str):
             val = "'{}'".format(val)
         elif isinstance(val, bool):
@@ -362,9 +409,14 @@ def print_example_options(f, cls, name, indent, po):
                 f.write(ind_str+'{}: {}\n'.format(k, val._default))
             else:
                 f.write(ind_str+'{}:\n'.format(k))
-                print_example_options(f, val, '', indent+2, None)
+                print_example_options(f, val, '', indent+2, None, help and 'list(dict' in help_lines[0])
         else:
+            if is_list and first:
+                k = '- '+k
             f.write(ind_str+'{}: {}\n'.format(k, val))
+            if is_list and first:
+                ind_str += '  '
+                first = False
     return obj
 
 
@@ -377,13 +429,19 @@ def create_example(pcb_file, out_dir, copy_options, copy_expand):
         exit(WONT_OVERWRITE)
     with open(fname, 'w') as f:
         logger.info('Creating {} example configuration'.format(fname))
+        f.write("# ATTENTION! THIS ISN'T A FULLY FUNCTIONAL EXAMPLE.\n")
+        f.write("# You should take portions of this example and edit the options to make\n")
+        f.write("# them suitable for your use.\n")
+        f.write("# This file is useful to know all the available options.\n")
         f.write('kibot:\n  version: 1\n')
         # Preflights
         f.write('\npreflight:\n')
         pres = BasePreFlight.get_registered()
         for n, o in OrderedDict(sorted(pres.items())).items():
             if o.__doc__:
-                f.write('  #'+o.__doc__.rstrip()+'\n')
+                lines = trim(o.__doc__.rstrip()+'.')
+                for ln in lines:
+                    f.write('  # '+ln.rstrip()+'\n')
             f.write('  {}: {}\n'.format(n, o.get_example()))
         # Outputs
         outs = RegOutput.get_registered()
@@ -401,7 +459,7 @@ def create_example(pcb_file, out_dir, copy_options, copy_expand):
         for n, cls in OrderedDict(sorted(outs.items())).items():
             lines = trim(cls.__doc__)
             if len(lines) == 0:
-                lines = ['Undocumented', 'No description']  # pragma: no cover
+                lines = ['Undocumented', 'No description']
             f.write('  # '+lines[0].rstrip()+':\n')
             for ln in range(2, len(lines)):
                 f.write('  # '+lines[ln].rstrip()+'\n')
