@@ -1,10 +1,9 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Salvador E. Tropea
-# Copyright (c) 2020 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2021 Salvador E. Tropea
+# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 import os
-import re
 from datetime import datetime
 from sys import exit
 from .misc import (EXIT_BAD_ARGS)
@@ -39,12 +38,14 @@ class GS(object):
     today = n.strftime('%Y-%m-%d')
     time = n.strftime('%H-%M-%S')
     kicad_version = ''
+    kicad_conf_path = None
+    kicad_share_path = None
+    kicad_plugins_dirs = []
     # KiCad version: major*1e6+minor*1e3+patch
     kicad_version_n = 0
     kicad_version_major = 0
     kicad_version_minor = 0
     kicad_version_patch = 0
-    board_comps_joined = False  # Flag to indicate we already merged data from the board
     # Data from the SCH because it doesn't have a Python API
     sch_title = None
     sch_date = None
@@ -55,6 +56,12 @@ class GS(object):
     pcb_date = None
     pcb_rev = None
     pcb_comp = None
+    # Current variant/s
+    variant = None
+    # All the outputs
+    outputs = None
+    # Name for the output we are generating
+    current_output = None
     # Global defaults
     #  This is used as default value for classes supporting "output" option
     def_global_output = '%f-%i%v.%x'
@@ -64,7 +71,10 @@ class GS(object):
     global_from_cli = {}
     global_output = None
     global_variant = None
+    global_kiauto_wait_start = None
+    global_kiauto_time_out_scale = None
     global_opts_class = None
+    test_boolean = True
 
     @staticmethod
     def set_sch(name):
@@ -88,37 +98,11 @@ class GS(object):
     def load_sch_title_block():
         if GS.sch_title is not None:
             return
-        GS.sch_title = ''
-        GS.sch_date = ''
-        GS.sch_rev = ''
-        GS.sch_comp = ''
-        re_val = re.compile(r"(\w+)\s+\"([^\"]+)\"")
-        with open(GS.sch_file) as f:
-            for line in f:
-                m = re_val.match(line)
-                if not m:
-                    if line.startswith("$EndDescr"):
-                        break
-                    # This line is executed, but coverage fails to detect it
-                    continue  # pragma: no cover
-                name, val = m.groups()
-                if name == "Title":
-                    GS.sch_title = val
-                elif name == "Date":
-                    GS.sch_date = val
-                elif name == "Rev":
-                    GS.sch_rev = val
-                elif name == "Comp":
-                    GS.sch_comp = val
-        if not GS.sch_date:
-            file_mtime = os.path.getmtime(GS.sch_file)
-            GS.sch_date = datetime.fromtimestamp(file_mtime).strftime('%Y-%m-%d_%H-%M-%S')
-        if not GS.sch_title:
-            GS.sch_title = GS.sch_basename
-        logger.debug("SCH title: `{}`".format(GS.sch_title))
-        logger.debug("SCH date: `{}`".format(GS.sch_date))
-        logger.debug("SCH revision: `{}`".format(GS.sch_rev))
-        logger.debug("SCH company: `{}`".format(GS.sch_comp))
+        assert GS.sch is not None
+        GS.sch_title = GS.sch.title
+        GS.sch_date = GS.sch.date
+        GS.sch_rev = GS.sch.revision
+        GS.sch_comp = GS.sch.company
 
     @staticmethod
     def load_pcb_title_block():
