@@ -23,12 +23,12 @@ import platform
 import sysconfig
 from ..gs import GS
 from .. import log
-from ..misc import W_NOHOME, W_NOUSER, W_BADSYS, W_NOCONFIG, W_NOKIENV, W_NOLIBS, W_NODEFSYMLIB
+from ..misc import W_NOCONFIG, W_NOKIENV, W_NOLIBS, W_NODEFSYMLIB
 
 # Check python version to determine which version of ConfirParser to import
 if sys.version_info.major >= 3:
     import configparser as ConfigParser
-else:  # pragma: no cover
+else:  # pragma: no cover (Py2)
     # For future Python 2 support
     import ConfigParser
 
@@ -123,54 +123,11 @@ class KiConf(object):
     def find_kicad_common():
         """ Looks for kicad_common config file.
             Returns its name or None. """
-        # User option has the higher priority
-        user_set = os.environ.get('KICAD_CONFIG_HOME')
-        if user_set:
-            cfg = os.path.join(user_set, KICAD_COMMON)
+        cfg = ''
+        if GS.kicad_conf_path:
+            cfg = os.path.join(GS.kicad_conf_path, KICAD_COMMON)
             if os.path.isfile(cfg):
                 return cfg
-        # XDG option is second
-        xdg_set = os.environ.get('XDG_CONFIG_HOME')
-        if xdg_set:
-            cfg = os.path.join(xdg_set, 'kicad', KICAD_COMMON)
-            if os.path.isfile(cfg):
-                return cfg
-        # Others depends on the OS
-        system = platform.system()
-        if system == 'Linux':
-            # Linux: ~/.config/kicad/
-            home = os.environ.get('HOME')
-            if not home:
-                logger.warning(W_NOHOME + 'Environment variable `HOME` not defined, using `/`')
-                home = '/'
-            cfg = os.path.join(home, '.config', 'kicad', KICAD_COMMON)
-            if os.path.isfile(cfg):
-                return cfg
-        elif system == 'Darwin':  # pragma: no cover
-            # MacOSX: ~/Library/Preferences/kicad/
-            home = os.environ.get('HOME')
-            if not home:
-                logger.warning(W_NOHOME + 'Environment variable `HOME` not defined, using `/`')
-                home = '/'
-            cfg = os.path.join(home, 'Library', 'Preferences', 'kicad', KICAD_COMMON)
-            if os.path.isfile(cfg):
-                return cfg
-        elif system == 'Windows':  # pragma: no cover
-            # Windows: C:\Users\username\AppData\Roaming\kicad
-            #      or  C:\Documents and Settings\username\Application Data\kicad
-            username = os.environ.get('username')
-            if not username:
-                logger.warning(W_NOUSER + 'Unable to determine current user')
-                return None
-            cfg = os.path.join('C:', 'Users', username, 'AppData', 'Roaming', 'kicad', KICAD_COMMON)
-            if os.path.isfile(cfg):
-                return cfg
-            cfg = os.path.join('C:', 'Documents and Settings', username, 'Application Data', 'kicad', KICAD_COMMON)
-            if os.path.isfile(cfg):
-                return cfg
-        else:  # pragma: no cover
-            logger.warning(W_BADSYS + 'Unsupported system `{}`'.format(system))
-            return None
         logger.warning(W_NOCONFIG + 'Unable to find KiCad configuration file ({})'.format(cfg))
         return None
 
@@ -194,7 +151,7 @@ class KiConf(object):
                 dir = os.path.join(sysconfig.get_path('data', 'posix_prefix'), share)
                 if os.path.isdir(dir):
                     return dir
-        elif system == 'Darwin':  # pragma: no cover
+        elif system == 'Darwin':  # pragma: no cover (Darwin)
             app_data = os.path.join('Library', 'Application Support', 'kicad', 'library')
             home = os.environ.get('HOME')
             if home:
@@ -204,7 +161,7 @@ class KiConf(object):
             dir = os.path.join('/', app_data)
             if os.path.isdir(dir):
                 return dir
-        elif system == 'Windows':  # pragma: no cover
+        elif system == 'Windows':  # pragma: no cover (Windows)
             dir = os.path.join('C:', 'Program Files', 'KiCad', share)
             if os.path.isdir(dir):
                 return dir
@@ -229,7 +186,10 @@ class KiConf(object):
             io_buf = StringIO('[Default]\n'+buf)
             cf = ConfigParser.RawConfigParser(allow_no_value=True)
             cf.optionxform = str
-            cf.readfp(io_buf, cfg)
+            if sys.version_info.major >= 3:
+                cf.read_file(io_buf, cfg)
+            else:  # pragma: no cover (Py2)
+                cf.readfp(io_buf, cfg)
             if 'EnvironmentVariables' not in cf.sections():
                 logger.warning(W_NOKIENV + 'KiCad config without EnvironmentVariables section')
             else:
@@ -250,7 +210,7 @@ class KiConf(object):
 
     def load_lib_aliases(fname):
         if not os.path.isfile(fname):
-            return
+            return False
         logger.debug('Loading symbols lib table `{}`'.format(fname))
         with open(fname, 'rt') as f:
             line = f.readline().strip()
@@ -269,13 +229,21 @@ class KiConf(object):
                     raise KiConfError('Unknown symbol table entry', SYM_LIB_TABLE, cline, line)
                 line = f.readline()
                 cline += 1
+        return True
 
     def load_all_lib_aliases():
         # Load the default symbol libs table.
         # This is the list of libraries enabled by the user.
+        loaded = False
         if KiConf.config_dir:
-            KiConf.load_lib_aliases(os.path.join(KiConf.config_dir, SYM_LIB_TABLE))
-        else:
+            conf_dir = KiConf.config_dir
+            if 'KICAD_CONFIG_HOME' in KiConf.kicad_env:
+                # KiCad 5 unintentionally allows it, is a bug, and won't be fixed:
+                # https://forum.kicad.info/t/kicad-config-home-inconsistencies-and-detail/26875
+                conf_dir = KiConf.kicad_env['KICAD_CONFIG_HOME']
+                logger.debug('Redirecting symbols lib table to '+conf_dir)
+            loaded = KiConf.load_lib_aliases(os.path.join(conf_dir, SYM_LIB_TABLE))
+        if not loaded:
             logger.warning(W_NODEFSYMLIB + 'Missing default symbol library table')
             # No default symbol libs table, try to create one
             if KiConf.sym_lib_dir:
@@ -288,3 +256,6 @@ class KiConf(object):
                     KiConf.lib_aliases[alias.name] = alias
         # Load the project's table
         KiConf.load_lib_aliases(os.path.join(KiConf.dirname, SYM_LIB_TABLE))
+
+    def expand_env(name):
+        return os.path.abspath(expand_env(un_quote(name), KiConf.kicad_env))
