@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Salvador E. Tropea
-# Copyright (c) 2020 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2021 Salvador E. Tropea
+# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 """
@@ -11,12 +11,14 @@ Currently oriented to collect the components for the BoM.
 # Encapsulate file/line
 import re
 import os
+from datetime import datetime
+from copy import deepcopy
 from collections import OrderedDict
 from .config import KiConf, un_quote
 from ..gs import GS
 from ..misc import (W_BADPOLI, W_POLICOORDS, W_BADSQUARE, W_BADCIRCLE, W_BADARC, W_BADTEXT, W_BADPIN, W_BADCOMP, W_BADDRAW,
                     W_UNKDCM, W_UNKAR, W_ARNOPATH, W_ARNOREF, W_MISCFLD, W_EXTRASPC, W_NOLIB, W_INCPOS, W_NOANNO, W_MISSLIB,
-                    W_MISSDCM, W_MISSCMP)
+                    W_MISSDCM, W_MISSCMP, W_MISFLDNAME)
 from .. import log
 
 logger = log.get_logger(__name__)
@@ -139,8 +141,11 @@ class LibComponentField(object):
             field.name = gs[9][1:-1]
         else:
             if field.number > 3:
-                raise SchLibError('Missing component field name', line, f)
-            field.name = ['Reference', 'Value', 'Footprint', 'Datasheet'][field.number]
+                logger.warning(W_MISFLDNAME + 'Missing component field name ({} line {})'.format(lib_name, f.line))
+                # KiCad falls-back to `FieldN`
+                field.name = 'Field'+str(field.number)
+            else:
+                field.name = ['Reference', 'Value', 'Footprint', 'Datasheet'][field.number]
         return field
 
     def write(self, f):
@@ -803,10 +808,16 @@ class SchematicComponent(object):
         self.footprint = ''
         self.datasheet = ''
         self.desc = ''
+        self.fields = []
+        self.dfields = {}
+        self.fields_bkp = None
+        self.dfields_bkp = None
         # Will be computed
         self.fitted = True
         self.included = True
         self.fixed = False
+        self.bottom = False
+        self.footprint_rot = 0.0
         # KiCad 5 PCB flags (mutually exclusive)
         self.smd = False
         self.virtual = False
@@ -818,6 +829,29 @@ class SchematicComponent(object):
             return self.dfields[field].value
         return ''
 
+    def get_free_field_number(self):
+        """ Looks for a field number that isn't currently in use """
+        max_num = -1
+        for f in self.fields:
+            if f.number > max_num:
+                max_num = f.number
+        return max_num+1
+
+    def set_field(self, field, value):
+        """ Change the value for an existing field """
+        if field in self.dfields:
+            target = self.dfields[field]
+            target.value = value
+            # Adjust special fields
+            if target.number < 4:
+                self._solve_fields(LineReader(None, '**Internal**'))
+        else:
+            f = SchematicField()
+            f.name = field
+            f.value = value
+            f.number = self.get_free_field_number()
+            self.add_field(f)
+
     # def get_field_names(self):
     #     return [f.name for f in self.fields]
 
@@ -829,6 +863,19 @@ class SchematicComponent(object):
         self.fields.append(field)
         self.dfields[field.name.lower()] = field
 
+    def back_up_fields(self):
+        """ First call makes a back-up of the fields.
+            Next calls restores the back-up. """
+        if self.fields_bkp:
+            # We have a back-up, restore from it
+            self.fields = deepcopy(self.fields_bkp)
+            self.dfields = {f.name.lower(): f for f in self.fields}
+            self._solve_fields(LineReader(None, '**Internal**'))
+        else:
+            # No back-up. Make one for the next reset
+            self.fields_bkp = deepcopy(self.fields)
+            self.dfields_bkp = {f.name.lower(): f for f in self.fields_bkp}
+
     def _solve_ref(self, path):
         """ Look fo the correct reference for this path.
             Returns the default reference if no paths defined.
@@ -889,12 +936,17 @@ class SchematicComponent(object):
                 field.value = stripped_val
 
     def __str__(self):
+        ref = self.ref
+        # Add the sub-part id
+        # How to know if unit 1 is A?
+        if self.unit > 1:
+            ref += chr(ord('A')+self.unit-1)
         if self.name == self.value:
-            return '{} ({})'.format(self.ref, self.name)
-        return '{} ({} {})'.format(self.ref, self.name, self.value)
+            return '{} ({})'.format(ref, self.name)
+        return '{} ({} {})'.format(ref, self.name, self.value)
 
     @staticmethod
-    def load(f, sheet_path, sheet_path_h, libs, fields, fields_lc):
+    def load(f, project, sheet_path, sheet_path_h, libs, fields, fields_lc):
         # L lib:name reference
         line = f.get_line()
         if not line or line[0] != 'L':
@@ -903,6 +955,7 @@ class SchematicComponent(object):
         if len(res) != 2:
             raise SchFileError('Malformed component label', line, f)
         comp = SchematicComponent()
+        comp.project = project
         comp.name, comp.f_ref = res
         res = comp.name.split(':')
         comp.lib = None
@@ -938,8 +991,6 @@ class SchematicComponent(object):
             comp.ar.append(SchematicAltRef.parse(line))
             line = f.get_line()
         # F field_number "text" orientation posX posY size Flags (see below) hjustify vjustify/italic/bold "name"
-        comp.fields = []
-        comp.dfields = {}
         while line[0] == 'F':
             field = SchematicField.parse(line, f)
             name_lc = field.name.lower()
@@ -959,11 +1010,11 @@ class SchematicComponent(object):
         # Redundant pos
         if not line.startswith('\t'+str(comp.unit)):
             raise SchFileError('Missing component redundant position', line, f)
-        res = _split_space(line[2:])
-        if len(res) != 2:
+        res = _split_space(line[1:])
+        if len(res) != 3:
             raise SchFileError('Malformed component redundant position', line, f)
-        xr = int(res[0])
-        yr = int(res[1])
+        xr = int(res[1])
+        yr = int(res[2])
         if comp.x != xr or comp.y != yr:
             logger.warning(W_INCPOS + 'Inconsistent position for component {} ({},{} vs {},{})'.
                            format(comp.f_ref, comp.x, comp.y, xr, yr))
@@ -1226,7 +1277,7 @@ class SchematicSheet(object):
         self.sheet = None
         self.id = ''
 
-    def load_sheet(self, parent, sheet_path, sheet_path_h, libs, fields, fields_lc):
+    def load_sheet(self, project, parent, sheet_path, sheet_path_h, libs, fields, fields_lc):
         assert self.name
         self.sheet = Schematic()
         parent_dir = os.path.dirname(parent)
@@ -1234,7 +1285,7 @@ class SchematicSheet(object):
         if len(sheet_path_h) > 1:
             sheet_path_h += '/'
         sheet_path_h += self.name if self.name else 'Unknown'
-        self.sheet.load(os.path.join(parent_dir, self.file), sheet_path, sheet_path_h, libs, fields, fields_lc)
+        self.sheet.load(os.path.join(parent_dir, self.file), project, sheet_path, sheet_path_h, libs, fields, fields_lc)
         return self.sheet
 
     @staticmethod
@@ -1317,6 +1368,10 @@ class Schematic(object):
         while True:
             line = f.get_line()
             if line.startswith('$EndDescr'):
+                self.title = self.title_block['Title'] if 'Title' in self.title_block else ''
+                self.date = self.title_block['Date'] if 'Date' in self.title_block else ''
+                self.revision = self.title_block['Rev'] if 'Rev' in self.title_block else ''
+                self.company = self.title_block['Comp'] if 'Comp' in self.title_block else ''
                 return
             elif line.startswith('encoding'):
                 if line[9:14] != 'utf-8':
@@ -1333,7 +1388,7 @@ class Schematic(object):
                     raise SchFileError('Wrong entry in title block', line, f)
                 self.title_block[m.group(1)] = m.group(2)
 
-    def load(self, fname, sheet_path='', sheet_path_h='/', libs={}, fields=[], fields_lc=set()):
+    def load(self, fname, project, sheet_path='', sheet_path_h='/', libs={}, fields=[], fields_lc=set()):
         """ Load a v5.x KiCad Schematic.
             The caller must be sure the file exists.
             Only the schematics are loaded not the libs. """
@@ -1342,6 +1397,7 @@ class Schematic(object):
         self.libs = libs
         self.fields = fields
         self.fields_lc = fields_lc
+        self.project = project
         with open(fname, 'rt') as fh:
             f = SCHLineReader(fh, fname)
             line = f.get_line()
@@ -1361,7 +1417,18 @@ class Schematic(object):
             line = f.get_line()
             if not line.startswith('EELAYER END'):
                 raise SchFileError('Missing EELAYER END', line, f)
+            # Load the title block
             self._get_title_block(f)
+            # Fill in some missing info
+            if not self.date:
+                file_mtime = os.path.getmtime(fname)
+                self.date = datetime.fromtimestamp(file_mtime).strftime('%Y-%m-%d_%H-%M-%S')
+            if not self.title:
+                self.title = os.path.splitext(os.path.basename(fname))[0]
+            logger.debug("SCH title: `{}`".format(self.title))
+            logger.debug("SCH date: `{}`".format(self.date))
+            logger.debug("SCH revision: `{}`".format(self.revision))
+            logger.debug("SCH company: `{}`".format(self.company))
             line = f.get_line()
             self.all = []
             self.components = []
@@ -1372,7 +1439,7 @@ class Schematic(object):
             self.sheets = []
             while not line.startswith('$EndSCHEMATC'):
                 if line.startswith('$Comp'):
-                    obj = SchematicComponent.load(f, sheet_path, sheet_path_h, libs, fields, fields_lc)
+                    obj = SchematicComponent.load(f, project, sheet_path, sheet_path_h, libs, fields, fields_lc)
                     self.components.append(obj)
                 elif line.startswith('NoConn'):
                     obj = SchematicConnection.parse(False, line[7:], f)
@@ -1399,21 +1466,21 @@ class Schematic(object):
             # Load sub-sheets
             self.sub_sheets = []
             for sch in self.sheets:
-                self.sub_sheets.append(sch.load_sheet(fname, sheet_path, sheet_path_h, libs, fields, fields_lc))
+                self.sub_sheets.append(sch.load_sheet(project, fname, sheet_path, sheet_path_h, libs, fields, fields_lc))
 
     def get_files(self):
-        """ A list of the names for all the sheets, including this one. """
-        files = [self.fname]
+        """ A list of the names for all the sheets, including this one.
+            We avoid repeating the same file. """
+        files = {self.fname}
         for sch in self.sheets:
-            files.extend(sch.sheet.get_files())
-        return files
+            files.update(sch.sheet.get_files())
+        return list(files)
 
     def get_components(self, exclude_power=True):
         """ A list of all the components. """
         if exclude_power:
             components = [c for c in self.components if not c.is_power]
-        else:  # pragma: no cover
-            # Currently unused
+        else:
             components = [c for c in self.components]
         for sch in self.sheets:
             components.extend(sch.sheet.get_components(exclude_power))
@@ -1552,3 +1619,15 @@ class Schematic(object):
             f.write(' (lib (name n)(type Legacy)(uri ${KIPRJMOD}/n.lib)(options "")(descr ""))\n')
             f.write(')\n')
         return fname
+
+    def file_names_variant(self, dest_dir):
+        """ Returns a list of file names created by save_variant() """
+        fnames = [os.path.join(dest_dir, 'y.lib'),
+                  os.path.join(dest_dir, 'n.lib'),
+                  os.path.join(dest_dir, 'sym-lib-table')]
+        # Sub-sheets
+        sub_sheets = self.get_files()
+        for sch in sub_sheets:
+            sch = os.path.basename(sch)
+            fnames.append(os.path.join(dest_dir, sch.replace('/', '_')))
+        return fnames
