@@ -1,12 +1,12 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Salvador E. Tropea
-# Copyright (c) 2020 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2021 Salvador E. Tropea
+# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 import pcbnew
 from .optionable import Optionable
 from .gs import GS
-from .misc import KICAD_VERSION_5_99
+from .misc import KICAD_VERSION_5_99, W_NOTASCII
 from re import match
 from .error import (PlotError, KiPlotConfigurationError)
 from .macros import macros, document, output_class  # noqa: F401
@@ -75,6 +75,20 @@ class Layer(Optionable):
         'F.CrtYd': 'F.Courtyard',
         'B.CrtYd': 'B.Courtyard',
     }
+    # Protel extensions
+    PROTEL_EXTENSIONS = {
+        pcbnew.F_Cu: 'gtl',
+        pcbnew.B_Cu: 'gbl',
+        pcbnew.F_Adhes: 'gta',
+        pcbnew.B_Adhes: 'gba',
+        pcbnew.F_Paste: 'gtp',
+        pcbnew.B_Paste: 'gbp',
+        pcbnew.F_SilkS: 'gto',
+        pcbnew.B_SilkS: 'gbo',
+        pcbnew.F_Mask: 'gts',
+        pcbnew.B_Mask: 'gbs',
+        pcbnew.Edge_Cuts: 'gm1',
+    }
     # Names from the board file
     _pcb_layers = None
     _plot_layers = None
@@ -87,8 +101,9 @@ class Layer(Optionable):
             self.suffix = ''
             """ Suffix used in file names related to this layer. Derived from the name if not specified """
             self.description = ''
-            """ A description for the layer, for documentation purposes """  # pragma: no cover
+            """ A description for the layer, for documentation purposes """
         self._unkown_is_error = True
+        self._protel_extension = None
 
     def config(self):
         super().config()
@@ -101,11 +116,33 @@ class Layer(Optionable):
                 self.description = 'No description'
         if not self.suffix:
             self.suffix = self.layer.replace('.', '_')
+        self.clean_suffix()
+
+    def clean_suffix(self):
+        filtered_suffix = ''.join(char for char in self.suffix if ord(char) < 128)
+        if filtered_suffix != self.suffix:
+            logger.warning(W_NOTASCII+'Only ASCII chars are allowed for layer suffixes ({}), using {}'.
+                           format(self, filtered_suffix))
+            self.suffix = filtered_suffix
 
     @property
     def id(self):
         return self._id
 
+    def fix_protel_ext(self):
+        """ Makes sure we have a defined Protel extension """
+        if self._protel_extension is not None:
+            # Already set, keep it
+            return
+        if self._is_inner:
+            self._protel_extension = 'g'+str(self.id-pcbnew.F_Cu+1)
+            return
+        if self.id in Layer.PROTEL_EXTENSIONS:
+            self._protel_extension = Layer.PROTEL_EXTENSIONS[self.id]
+            return
+        self._protel_extension = 'gbr'
+        return
+
     @staticmethod
     def solve(values):
         board = GS.board
@@ -135,6 +172,7 @@ class Layer(Optionable):
                     # Check if the layer is in use
                     if layer._is_inner and (layer._id < 1 or layer._id >= layer_cnt - 1):
                         raise PlotError("Inner layer `{}` is not valid for this board".format(layer))
+                    layer.fix_protel_ext()
                     new_vals.append(layer)
                 else:  # A string
                     ext = None
@@ -151,13 +189,15 @@ class Layer(Optionable):
                     elif layer in Layer._pcb_layers:
                         ext = [Layer.create_layer(layer)]
                     # Give compatibility for the KiCad 5 default names (automagically renamed by KiCad 6)
-                    elif GS.kicad_version_n >= KICAD_VERSION_5_99 and layer in Layer.KICAD6_RENAME:
+                    elif GS.kicad_version_n >= KICAD_VERSION_5_99 and layer in Layer.KICAD6_RENAME:  # pragma: no cover (Ki6)
                         ext = [Layer.create_layer(Layer.KICAD6_RENAME[layer])]
+                    elif layer in Layer.DEFAULT_LAYER_NAMES:
+                        ext = [Layer.create_layer(layer)]
                     if ext is None:
                         raise KiPlotConfigurationError("Unknown layer spec: `{}`".format(layer))
                     new_vals.extend(ext)
             return new_vals
-        assert False, "Unimplemented layer type "+str(type(values))  # pragma: no cover
+        assert False, "Unimplemented layer type "+str(type(values))
 
     @staticmethod
     def _get_copper():
@@ -182,6 +222,8 @@ class Layer(Optionable):
         layer.suffix = name.replace('.', '_')
         layer.description = Layer.DEFAULT_LAYER_DESC.get(name)
         layer._get_layer_id_from_name()
+        layer.fix_protel_ext()
+        layer.clean_suffix()
         return layer
 
     @staticmethod
@@ -211,7 +253,7 @@ class Layer(Optionable):
             if id is not None:
                 # 2) List from the PCB
                 self._id = id
-                self._is_inner = id < pcbnew.B_Cu
+                self._is_inner = id > pcbnew.F_Cu and id < pcbnew.B_Cu
             elif self.layer.startswith("Inner"):
                 # 3) Inner.N names
                 m = match(r"^Inner\.([0-9]+)$", self.layer)
@@ -224,4 +266,12 @@ class Layer(Optionable):
         return self._id
 
     def __str__(self):
-        return "{} ({} '{}' {})".format(self.layer, self._id, self.description, self.suffix)
+        if hasattr(self, '_id'):
+            return "{} ({} '{}' {})".format(self.layer, self._id, self.description, self.suffix)
+        return "{} ('{}' {})".format(self.layer, self.description, self.suffix)
+
+
+for i in range(1, 30):
+    name = 'In'+str(i)+'.Cu'
+    Layer.DEFAULT_LAYER_NAMES[name] = pcbnew.In1_Cu+i-1
+    Layer.DEFAULT_LAYER_DESC[name] = 'Inner layer '+str(i)
