@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Salvador E. Tropea
-# Copyright (c) 2020 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2021 Salvador E. Tropea
+# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 # Adapted from: https://stackoverflow.com/questions/384076/how-can-i-color-python-logging-output
@@ -11,15 +11,14 @@ Handles logging initialization and formating.
 """
 import sys
 import logging
-from io import StringIO
 no_colorama = False
 try:
     from colorama import init as colorama_init, Fore, Back, Style
-except ImportError:  # pragma: no cover
+except ImportError:
     no_colorama = True
 # If colorama isn't installed use an ANSI basic replacement
 if no_colorama:
-    from .ansi import Fore, Back, Style  # noqa: F811
+    from .mcpyrate.ansi import Fore, Back, Style  # noqa: F811
 else:
     colorama_init()
 # Default domain, base name for the tool
@@ -55,12 +54,13 @@ class MyLogger(logging.Logger):
     def warning(self, msg, *args, **kwargs):
         MyLogger.warn_tcnt += 1
         # Get the message applying optional C style expansions
-        if isinstance(msg, str) and len(args):
-            buf = StringIO()
-            buf.write(msg % args)
-            buf = buf.getvalue()
-        else:
-            buf = str(msg)
+        # No longer used:
+        # if isinstance(msg, str) and len(args):
+        #     buf = StringIO()
+        #     buf.write(msg % args)
+        #     buf = buf.getvalue()
+        # else:
+        buf = str(msg)
         # Avoid repeated warnings
         if buf in MyLogger.warn_hash:
             MyLogger.warn_hash[buf] += 1
@@ -77,7 +77,7 @@ class MyLogger(logging.Logger):
         MyLogger.warn_cnt += 1
         MyLogger.warn_hash[buf] = 1
         if sys.version_info.major > 3 or (sys.version_info.major == 3 and sys.version_info.minor >= 8):
-            super().warning(buf, stacklevel=2, **kwargs)
+            super().warning(buf, stacklevel=2, **kwargs)  # pragma: no cover (Py38)
         else:
             super().warning(buf, **kwargs)
 
@@ -99,13 +99,32 @@ def set_verbosity(logger, verbose, quiet):
     logger.setLevel(log_level)
 
 
+class FilterOnlyInfo(object):
+    def filter(self, record):
+        return record.levelno == logging.INFO
+
+
+class FilterNoInfo(object):
+    def filter(self, record):
+        return record.levelno != logging.INFO
+
+
 def init():
     """Initialize the logging feature using a custom format"""
     # Use a class to count and filter warnings
     logging.setLoggerClass(MyLogger)
     logger = get_logger()
+    # Handler for all but info.
+    # Outputs to stderr
     ch = logging.StreamHandler()
-    ch.setFormatter(CustomFormatter())
+    ch.addFilter(FilterNoInfo())
+    ch.setFormatter(CustomFormatter(sys.stderr))
+    logger.addHandler(ch)
+    # Handler for t info.
+    # Outputs to stdout
+    ch = logging.StreamHandler(sys.stdout)
+    ch.addFilter(FilterOnlyInfo())
+    ch.setFormatter(CustomFormatter(sys.stdout))
     logger.addHandler(ch)
     return logger
 
@@ -113,32 +132,34 @@ def init():
 class CustomFormatter(logging.Formatter):
     """Logging Formatter to add colors"""
 
-    if sys.stderr.isatty():
-        white = Fore.WHITE
-        yellow = Fore.YELLOW + Style.BRIGHT
-        red = Fore.RED + Style.BRIGHT
-        red_alarm = Fore.RED + Back.WHITE + Style.BRIGHT
-        cyan = Fore.CYAN + Style.BRIGHT
-        reset = Style.RESET_ALL
-    else:
-        white = ""
-        yellow = ""
-        red = ""
-        red_alarm = ""
-        cyan = ""
-        reset = ""
-    # format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s "
-    #          "(%(filename)s:%(lineno)d)"
-    format = "%(levelname)s:%(message)s (%(name)s - %(filename)s:%(lineno)d)"
-    format_simple = "%(message)s"
+    def __init__(self, stream):
+        super().__init__()
+        if stream.isatty():
+            white = Fore.WHITE
+            yellow = Fore.YELLOW + Style.BRIGHT
+            red = Fore.RED + Style.BRIGHT
+            red_alarm = Fore.RED + Back.WHITE + Style.BRIGHT
+            cyan = Fore.CYAN + Style.BRIGHT
+            reset = Style.RESET_ALL
+        else:
+            white = ""
+            yellow = ""
+            red = ""
+            red_alarm = ""
+            cyan = ""
+            reset = ""
+        # format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s "
+        #          "(%(filename)s:%(lineno)d)"
+        format = "%(levelname)s:%(message)s (%(name)s - %(filename)s:%(lineno)d)"
+        format_simple = "%(message)s"
 
-    FORMATS = {
-        logging.DEBUG: cyan + format + reset,
-        logging.INFO: white + format_simple + reset,
-        logging.WARNING: yellow + format + reset,
-        logging.ERROR: red + format + reset,
-        logging.CRITICAL: red_alarm + format + reset
-    }
+        self.FORMATS = {
+            logging.DEBUG: cyan + format + reset,
+            logging.INFO: white + format_simple + reset,
+            logging.WARNING: yellow + format + reset,
+            logging.ERROR: red + format + reset,
+            logging.CRITICAL: red_alarm + format + reset
+        }
 
     def format(self, record):
         log_fmt = self.FORMATS.get(record.levelno)
