@@ -1,10 +1,15 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Salvador E. Tropea
-# Copyright (c) 2020 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2021 Salvador E. Tropea
+# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 """ Miscellaneous definitions """
 
+import re
+import os
+from contextlib import contextmanager
+
+
 # Error levels
 INTERNAL_ERROR = 1    # Unhandled exceptions
 WRONG_ARGUMENTS = 2   # This is what argsparse uses
@@ -67,12 +72,14 @@ URL_IBOM = 'https://github.com/INTI-CMNB/InteractiveHtmlBom'
 KICAD2STEP = 'kicad2step'
 PCBDRAW = 'pcbdraw'
 URL_PCBDRAW = 'https://github.com/INTI-CMNB/pcbdraw'
-EXAMPLE_CFG = 'example.kibot.yaml'
+EXAMPLE_CFG = 'example_template.kibot.yaml'
 AUTO_SCALE = 0
 KICAD_VERSION_5_99 = 5099000
 
 # Internal filter names
-IFILL_MECHANICAL = '_mechanical'
+IFILT_MECHANICAL = '_mechanical'
+IFILT_VAR_RENAME = '_var_rename'
+IFILT_ROT_FOOTPRINT = '_rot_footprint'
 # KiCad 5 GUI values for the attribute
 UI_THT = 0       # 1 for KiCad 6
 UI_SMD = 1       # 2 for KiCad 6
@@ -156,6 +163,18 @@ W_MISSDCM = '(W042) '
 W_MISSCMP = '(W043) '
 W_VARSCH = '(W044) '
 W_WRONGPASTE = '(W045) '
+W_MISFLDNAME = '(W046) '
+W_MISS3D = '(W047) '
+W_FAILDL = '(W048) '
+W_NOLAYER = '(W049) '
+W_EMPTYZIP = '(W050) '
+W_WRONGCHAR = '(W051) '
+W_NOKIVER = '(W052) '
+W_EXTNAME = '(W053) '
+W_TIMEOUT = '(W054) '
+W_MUSTBEINT = '(W055) '
+W_NOOUTPUTS = '(W056) '
+W_NOTASCII = '(W057) '
 
 
 class Rect(object):
@@ -178,3 +197,18 @@ class Rect(object):
             self.y1 = min(self.y1, wxRect.y)
             self.x2 = max(self.x2, wxRect.x+wxRect.width)
             self.y2 = max(self.y2, wxRect.y+wxRect.height)
+
+
+def name2make(name):
+    return re.sub(r'[ \$\.\\\/]', '_', name)
+
+
+@contextmanager
+def hide_stderr():
+    """ Low level stderr supression, used to hide KiCad bugs. """
+    newstderr = os.dup(2)
+    devnull = os.open('/dev/null', os.O_WRONLY)
+    os.dup2(devnull, 2)
+    os.close(devnull)
+    yield
+    os.dup2(newstderr, 2)
