@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Salvador E. Tropea
-# Copyright (c) 2020 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2021 Salvador E. Tropea
+# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 """ Base class for output options """
@@ -30,6 +30,7 @@ class Optionable(object):
         self._unkown_is_error = False
         self._error_context = ''
         self._tree = {}
+        self._configured = False
         super().__init__()
         if GS.global_output is not None and getattr(self, 'output', None):
             setattr(self, 'output', GS.global_output)
@@ -65,7 +66,10 @@ class Optionable(object):
             raise KiPlotConfigurationError("Option `{}` must be true/false".format(key))
 
     def get_doc(self, name):
-        doc = getattr(self, '_help_'+name).strip()
+        try:
+            doc = getattr(self, '_help_'+name).strip()
+        except AttributeError:
+            return None, name, False
         if doc[0] == '{':
             alias = doc[1:-1]
             return getattr(self, '_help_'+alias).strip(), alias, True
@@ -132,7 +136,7 @@ class Optionable(object):
                     if isinstance(v, (int, float)) and not isinstance(v, bool):
                         # Note: booleans are also instance of int
                         # Not used yet
-                        Optionable._check_num(k, v, cur_doc)  # pragma: no cover
+                        Optionable._check_num(k, v, cur_doc)  # pragma: no cover (Internal)
                     elif isinstance(v, str):
                         Optionable._check_str(k, v, cur_doc)
                     elif isinstance(v, dict):
@@ -165,8 +169,9 @@ class Optionable(object):
         self._tree = tree
 
     def config(self):
-        if self._tree:
+        if self._tree and not self._configured:
             self._perform_config_mapping()
+            self._configured = True
 
     def get_attrs_for(self):
         """ Returns all attributes """
@@ -193,12 +198,13 @@ class Optionable(object):
             name = name.replace('%c', GS.pcb_comp)
             name = name.replace('%d', GS.pcb_date)
             name = name.replace('%D', GS.today)
+            name = name.replace('%F', GS.pcb_no_ext)
             name = name.replace('%f', GS.pcb_basename)
             name = name.replace('%i', id)
             name = name.replace('%p', GS.pcb_title)
             name = name.replace('%r', GS.pcb_rev)
             name = name.replace('%T', GS.time)
-            name = name.replace('%v', self._find_variant())
+            name = name.replace('%v', self._find_variant() if self else '')
             name = name.replace('%x', ext)
             # sanitize the name to avoid characters illegal in file systems
             name = name.replace('\\', '/')
@@ -214,12 +220,13 @@ class Optionable(object):
             name = name.replace('%c', GS.sch_comp)
             name = name.replace('%d', GS.sch_date)
             name = name.replace('%D', GS.today)
+            name = name.replace('%F', GS.sch_no_ext)
             name = name.replace('%f', GS.sch_basename)
             name = name.replace('%i', id)
             name = name.replace('%p', GS.sch_title)
             name = name.replace('%r', GS.sch_rev)
             name = name.replace('%T', GS.time)
-            name = name.replace('%v', self._find_variant())
+            name = name.replace('%v', self._find_variant() if self else '')
             name = name.replace('%x', ext)
             # sanitize the name to avoid characters illegal in file systems
             name = name.replace('\\', '/')
