@@ -1,8 +1,13 @@
+# -*- coding: utf-8 -*-
+# Copyright (c) 2020-2021 Salvador E. Tropea
+# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
+# License: GPL-3.0
+# Project: KiBot (formerly KiPlot)
 import os
 from subprocess import (check_output, STDOUT, CalledProcessError)
-from .misc import (CMD_IBOM, URL_IBOM, BOM_ERROR)
+from .misc import (CMD_IBOM, URL_IBOM, BOM_ERROR, W_EXTNAME)
 from .gs import (GS)
-from .kiplot import check_script
+from .kiplot import check_script, search_as_plugin
 from .out_base import VariantOptions
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
@@ -54,7 +59,8 @@ class IBoMOptions(VariantOptions):
             self.sort_order = 'C,R,L,D,U,Y,X,F,SW,A,~,HS,CNN,J,P,NT,MH'
             """ Default sort order for components. Must contain '~' once """
             self.netlist_file = ''
-            """ Path to netlist or xml file """
+            """ Path to netlist or xml file. You can use '%F.xml' to avoid specifying the project name.
+                Leave it blank for most uses, data will be extracted from the PCB """
             self.extra_fields = ''
             """ Comma separated list of extra fields to pull from netlist or xml file """
             self.normalize_field_case = False
@@ -85,13 +91,31 @@ class IBoMOptions(VariantOptions):
         self.add_to_doc('variant', WARNING_MIX)
         self.add_to_doc('dnf_filter', WARNING_MIX)
 
-    def run(self, output_dir, board):
-        super().run(output_dir, board)
-        check_script(CMD_IBOM, URL_IBOM)
+    def config(self):
+        super().config()
+        self.netlist_file = self.expand_filename('', self.netlist_file, 'ibom', 'xml')
+
+    def get_targets(self, parent, out_dir):
+        if self.output:
+            return [self.expand_filename(out_dir, self.output, 'ibom', 'html')]
+        logger.warning(W_EXTNAME+'{} uses a name generated by the external tool.'.format(parent))
+        logger.warning(W_EXTNAME+'Please use a name generated by KiBot or specify the name explicitly.')
+        return []
+
+    def get_dependencies(self):
+        files = [GS.pcb_file]
+        if os.path.isfile(self.netlist_file):
+            files.append(self.netlist_file)
+        return files
+
+    def run(self, output_dir):
+        super().run(output_dir)
+        tool = search_as_plugin(CMD_IBOM, ['InteractiveHtmlBom', 'InteractiveHtmlBom/InteractiveHtmlBom'])
+        check_script(tool, URL_IBOM)
         logger.debug('Doing Interactive BoM')
         # Tell ibom we don't want to use the screen
         os.environ['INTERACTIVE_HTML_BOM_NO_DISPLAY'] = ''
-        cmd = [CMD_IBOM, GS.pcb_file, '--dest-dir', output_dir, '--no-browser', ]
+        cmd = [tool, GS.pcb_file, '--dest-dir', output_dir, '--no-browser', ]
         # Solve the output name
         output = None
         if self.output:
@@ -140,3 +164,6 @@ class IBoM(BaseOutput):  # noqa: F821
         with document:
             self.options = IBoMOptions
             """ [dict] Options for the `ibom` output """
+
+    def get_dependencies(self):
+        return self.options.get_dependencies()
