@@ -1,17 +1,18 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Salvador E. Tropea
-# Copyright (c) 2020 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2021 Salvador E. Tropea
+# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 import os
 from re import search
 from tempfile import NamedTemporaryFile
 from subprocess import (check_output, STDOUT, CalledProcessError)
-from .misc import (CMD_KIBOM, URL_KIBOM, BOM_ERROR)
+from .misc import (CMD_KIBOM, URL_KIBOM, BOM_ERROR, W_EXTNAME)
 from .kiplot import (check_script)
 from .gs import (GS)
 from .optionable import Optionable, BaseOptions
 from .error import KiPlotConfigurationError
+from .bom.columnlist import ColumnList
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
 
@@ -33,7 +34,7 @@ class KiBoMRegex(Optionable):
             self.field = None
             """ {column} """
             self.regexp = None
-            """ {regex} """  # pragma: no cover
+            """ {regex} """
 
     def __str__(self):
         return self.column+'\t'+self.regex
@@ -50,7 +51,9 @@ class KiBoMColumns(Optionable):
             self.name = ''
             """ Name to display in the header. The field is used when empty """
             self.join = Optionable
-            """ [list(string)|string=''] List of fields to join to this column """  # pragma: no cover
+            """ [list(string)|string=''] List of fields to join to this column """
+        self._field_example = 'Row'
+        self._name_example = 'Line'
 
     def config(self):
         super().config()
@@ -157,7 +160,7 @@ class KiBoMConfig(Optionable):
                 ..regex: 'fiducial' """
             self.columns = KiBoMColumns
             """ [list(dict)|list(string)] List of columns to display.
-                Can be just the name of the field """  # pragma: no cover
+                Can be just the name of the field """
 
     @staticmethod
     def _create_minimal_ini():
@@ -175,6 +178,8 @@ class KiBoMConfig(Optionable):
     @staticmethod
     def _get_columns():
         """ Create a list of valid columns """
+        if not GS.sch:
+            return ColumnList.COLUMNS_DEFAULT
         check_script(CMD_KIBOM, URL_KIBOM, '1.8.0')
         config = None
         csv = None
@@ -350,7 +355,14 @@ class KiBoMOptions(BaseOptions):
             self.conf.save(conf)
             self.conf = conf
 
-    def run(self, output_dir, board):
+    def get_targets(self, parent, out_dir):
+        if self.output:
+            return [self.expand_filename_sch(out_dir, self.output, 'bom', self.format.lower())]
+        logger.warning(W_EXTNAME+'{} uses a name generated by the external tool.'.format(parent))
+        logger.warning(W_EXTNAME+'Please use a name generated by KiBot or specify the name explicitly.')
+        return []
+
+    def run(self, output_dir):
         check_script(CMD_KIBOM, URL_KIBOM, '1.8.0')
         format = self.format.lower()
         prj = GS.sch_no_ext
@@ -405,3 +417,9 @@ class KiBoM(BaseOutput):  # noqa: F821
             self.options = KiBoMOptions
             """ [dict] Options for the `kibom` output """
         self._sch_related = True
+
+    def get_dependencies(self):
+        files = super().get_dependencies()
+        if isinstance(self.options.conf, str):
+            files.append(self.options.conf)
+        return files
