@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Salvador E. Tropea
-# Copyright (c) 2020 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2021 Salvador E. Tropea
+# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
 # Copyright (c) 2020 @nerdyscout
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
@@ -8,7 +8,7 @@ import os
 from tempfile import mkdtemp
 from shutil import rmtree
 from .gs import (GS)
-from .kiplot import check_eeschema_do, exec_with_retry
+from .kiplot import check_eeschema_do, exec_with_retry, add_extra_options
 from .misc import (CMD_EESCHEMA_DO, SVG_SCH_PRINT)
 from .out_base import VariantOptions
 from .macros import macros, document, output_class  # noqa: F401
@@ -21,12 +21,19 @@ class SVG_Sch_PrintOptions(VariantOptions):
     def __init__(self):
         with document:
             self.output = GS.def_global_output
-            """ filename for the output SVG (%i=schematic %x=svg) """
+            """ Filename for the output SVG (%i=schematic %x=svg) """
         super().__init__()
         self.add_to_doc('variant', "Not fitted components are crossed")
 
-    def run(self, output_dir, board):
-        super().run(output_dir, board)
+    def get_targets(self, parent, out_dir):
+        id = 'schematic'
+        ext = 'svg'
+        if self.output:
+            return [self.expand_filename_sch(out_dir, self.output, id, ext)]
+        return [self.expand_filename_sch(out_dir, '%f.%x', id, ext)]
+
+    def run(self, output_dir):
+        super().run(output_dir)
         check_eeschema_do()
         if self._comps:
             # Save it to a temporal dir
@@ -37,9 +44,7 @@ class SVG_Sch_PrintOptions(VariantOptions):
             sch_dir = None
             sch_file = GS.sch_file
         cmd = [CMD_EESCHEMA_DO, 'export', '--all_pages', '--file_format', 'svg', sch_file, output_dir]
-        if GS.debug_enabled:
-            cmd.insert(1, '-vv')
-            cmd.insert(1, '-r')
+        cmd, video_remove = add_extra_options(cmd)
         ret = exec_with_retry(cmd)
         if ret:
             logger.error(CMD_EESCHEMA_DO+' returned %d', ret)
@@ -55,6 +60,10 @@ class SVG_Sch_PrintOptions(VariantOptions):
         if sch_dir:
             logger.debug('Removing temporal variant dir `{}`'.format(sch_dir))
             rmtree(sch_dir)
+        if video_remove:
+            video_name = os.path.join(GS.out_dir, 'export_eeschema_screencast.ogv')
+            if os.path.isfile(video_name):
+                os.remove(video_name)
 
 
 @output_class
